/////////////////////////////////////////////////////////////////////////////////////////
//  author: qingzhe huang
//  date: oct. 4, 2006
//	purpose of program: demo basic usage of UDP socket in java.
//	1. This is a simple UDP server which binds a fixed port number.
//	2. DatagramSocket uses a DatagramPacket to receive message. DatagramPacket uses an 
//		internal buffer (a byte[]) to hold incoming message.
//	3. Since "byte[]" in jave will not be automatically converted into "char[]", you
//		have to give the "length" of incoming message to correctly convert into String.
//	4. In order to show UDP is not a reliable socket communication, you can change "quit" message
//		to a big number say 200000, see below. And probably you will observe the lost packet situation.
//////////////////////////////////////////////////////////////////////////////////////////

import java.net.*;

class MyServer
{
	public static final int ServerPortNumber=8000;
	public static final int MaxMsgLength=256;
	private static DatagramSocket server;
	private static byte[] buffer=new byte[MaxMsgLength];
	private static DatagramPacket packet=new DatagramPacket(buffer, MaxMsgLength);


	public static void main(String args[])
	{
		/////////////////////////////////////////
		//for this small number, Java UDP can work properly
		String quitMsg=new String("2000");//for this number the java UDP socket can manage


		//////////////////////////////////////////////
		//but if you change to use this big number, you may observe the lost packet in server.
		//String quitMsg=new String("200000");//for this number the java UDP socket probably loses the packet
		
		try
		{
			server=new DatagramSocket(ServerPortNumber);
		
			while (true)
			{
				server.receive(packet);
				String msg=new String(buffer, 0, packet.getLength());
				//Thread.sleep(100);
							
				System.out.println("message received:"+msg);
				if (msg.compareTo(quitMsg)==0)
				{
					break;
				}
			}
		}
		catch(Exception er)
		{
			System.out.println(er);
		}
	}
}

